# gitea.RdzeN.net
# HP Smart Tank Plus 555 head clean.
#

# requere installed snmp

# set time when printer can run head cleanning
$running_time_start = 10
$running_time_stop  = 20

# set day interval between last print and head cleannig 
$days_between = 7
$sec_per_day  = 86400
$days_between = $days_between * $sec_per_day

$location        = Get-Location
$path_last_run   = "$($location)/hp_clean_last_run"
$path_log        = "$($location)/hp_clean_log"
$snmp_ver        = "2c"
$snmp_community  = "public"
$snmp_code       = "iso.3.6.1.2.1.43.10.2.1.4.1.1" # OID for printed pages durring actuall sesion
$printer_ip      = "" # printer IP
$url             = "http://$($printer_ip)/DevMgmt/InternalPrintDyn.xml" # url where we need to POST body for start HEAD cleaning.
$body            = "<ipdyn:InternalPrintDyn xmlns:ipdyn=`"http://www.hp.com/schemas/imaging/con/ledm/internalprintdyn/2008/03/21`" xmlns:copy=`"http://www.hp.com/schemas/imaging/con/copy/2008/07/07`" xmlns:dd=`"http://www.hp.com/schemas/imaging/con/dictionaries/1.0/`" xmlns:dd3=`"http://www.hp.com/schemas/imaging/con/dictionaries/2009/04/06`" xmlns:fw=`"http://www.hp.com/schemas/imaging/con/firewall/2011/01/05`"><ipdyn:JobType>cleaningVerificationPage</ipdyn:JobType></ipdyn:InternalPrintDyn>" 
$headers        = @{
    "Content-Type" = "text/xml"
}

$logs           = @()
$log_limit      = 10000
function log {
    param (
        $l_level,
        $l_message
    )
    $l_timestamp = Get-Date -Format "yyyy-MM-dd HH:mm:ss.ffff K"
    switch ($l_level) {
        0 { 
            $log_message = "$($l_timestamp) | [INFO] | $($l_message)"
        }
        1 { 
            $log_message = "$($l_timestamp) | [WARNING] | $($l_message)"
        }
        2 { 
            $l_error = $Error[0].Exception
            $log_message = "$($l_timestamp) | [ERROR] | $($l_message) | $l_error"
        }
    }
    return $log_message
}
function log_store {
    $logs >> $path_log
    if (Test-Path $path_log) {
        $tmp = Get-Content $path_log -Tail $log_limit
        $tmp | Set-Content $path_log
    }
}
$logs += log -l_level 0 -l_message "Script Started"

$actual_time = get-date

# check if hour of running isnt restricted by $running_time_start and $running_time_stop
if (($actual_time.hour -lt $running_time_start) -and ($actual_time.hour -gt $running_time_stop)) {
    $logs += log -l_level 0 -l_message "Hours between >= $running_time_start and <= $running_time_stop are restricted. EXIT"
    $logs += log -l_level 0 -l_message "Script Stopped"
    log_store
    exit
}

$actual_time = get-date -Format "yyyy-MM-dd HH:mm:ss.ffff"

# check / create last run control file
if (-not(Test-Path $path_last_run)) {
    $logs += log -l_level 1 -l_message "$($path_last_run) Does not exists - CREATING."
    $last_run = [PSCustomObject]@{
        date  = Get-Date -Format "yyyy-MM-dd HH:mm:ss.ffff"
        p_count = (snmpget -v $snmp_ver -c $snmp_community $printer_ip $snmp_code).Split(" ")[-1]
    }
    $logs += log -l_level 0 -l_message "Actual date and printed documents count are stored in $($path_last_run)"
    $last_run | Export-Csv -Path $path_last_run -NoTypeInformation
# Get last run date and amount of printed documents
} else {
    $last_run = Import-Csv $path_last_run
    $logs += log -l_level 0 -l_message "Last Run Date: $($last_run.date), Last printed documents count: $($last_run.p_count)"
}

# Get actuall amount of printed pages
try {
    $actual_p_count = (snmpget -v $snmp_ver -c $snmp_community $printer_ip $snmp_code).Split(" ")[-1]
} catch {
    $logs += log -l_level 2 -l_message "Getting printed page count got ERROR. EXIT"
    $logs += log -l_level 0 -l_message "Script Stopped"
    log_store
    exit
}
# Check if printed page count changed
if ($actual_p_count -gt $last_run.p_count) {
    $logs += log -l_level 0 -l_message "Printed page count changed from $($last_run.p_count) to $actual_p_count. Changing last run time from  $($last_run.date) to $actual_time"
    $last_run.p_count = $actual_p_count
    $last_run.date    = $actual_time
    $last_run | Export-Csv -Path $path_last_run -NoTypeInformation
    $logs += log -l_level 0 -l_message "Script Stopped"
    log_store
    exit
}

# Check if between $last_run.date and $actual_time passed $days_between
$difference = [DateTime]$actual_time - [DateTime]$last_run.Date | Select-Object TotalSeconds -ExpandProperty TotalSeconds

if ($difference -gt $days_between) {
    $logs += log -l_level 0 -l_message "Difference between Actual Time and Last Run Time is greater than $($days_between/$sec_per_day) days, is it: $($difference/$sec_per_day) days"
    # start head cleaning
    try {
        $tmp = Invoke-RestMethod -Method Post -Body $body -Headers $headers -Uri $url
            $logs += log -l_level 0 -l_message "Head Cleaning Started"
            $actual_time        = get-date -Format "yyyy-MM-dd HH:mm:ss.ffff"
            $last_run.date      = $actual_time
    } catch {
            $logs += log -l_level 2 -l_message "Error durring connecting to printer."
            $logs += log -l_level 0 -l_message "Script Stopped"
            log_store
            exit
    } 
} else {
    $logs += log -l_level 0 -l_message "Difference between Actual Time and Last Run Time is less than $($days_between/$sec_per_day) days, is it: $($difference/$sec_per_day) days"
}

# store actuall amount of printed pages and actuall date
$actual_p_count     = (snmpget -v $snmp_ver -c $snmp_community $printer_ip $snmp_code).Split(" ")[-1]
$last_run.p_count   = $actual_p_count

$last_run | Export-Csv -Path $path_last_run -NoTypeInformation

$logs += log -l_level 0 -l_message "Head Cleaning Stopped. Actual date: $actual_time. Actuall printed page count: $actual_p_count."
$logs += log -l_level 0 -l_message "Script Stopped"
log_store
exit
